app.controller('treemapCtrl', ['$scope', '$rootScope', '$log', '$tm1Ui', '$ngBootbox','screenSize','$timeout', '$location', '$window', function ($scope, $rootScope, $log, $tm1Ui, $ngBootbox, screenSize, $timeout, $location, $window) {
    ////Sorting
    
    /*
     *     defaults.* are variables that are declared once and are changed in the page, otherwise known as constants in programming languages
     *     lists.* should be used to store any lists that are used with ng-repeat, i.e. tm1-ui-element-list
     *     selections.* should be used for all selections that are made by a user in the page
     *     values.* should store the result of any dbr, dbra or other values from server that you want to store to use elsewhere, i.e. in a calculation
     * 
     *     For more information: https://github.com/cubewise-code/canvas-best-practice
     */

    $scope.lists = {};
    $scope.values = {};

    $scope.screenSize = screenSize;
    

    // Default values to be used before selections are made
    $scope.defaults = {
        region: 'Total Europe',
        department: 'Corporate',
        measure: 'Order Quantity',
        version: 'Actual',
        product: 'Accessories',
        month: 'Year',
        year: '2015',
        OperatingProfit: 'Operating Profit',
        GrossMargin: 'Gross Margin',
        OperatingExpenses: 'Operating Expenses',
        OtherIncome: 'Other Income and Expense'
    }

    $scope.selections = {
        showContent: 'table',
        month: $scope.defaults.month,
        year: $scope.defaults.year,
        version: $scope.defaults.version,
        region: $scope.defaults.region,
        department: $scope.defaults.department,
        product: $scope.defaults.product,
        measure: $scope.defaults.measure,
        segment: 'region',
        subset: [
            {
                description: "Operating Profit",
                icon: ""
            },
            {
                description: "Gross Margin"
            },
            {
                description: "Operating Expenses"
            },
            {
                description: "Other Income and Expense"
            }
		]
    };
 
    //################//
    // START TREEMAP  //
    
    //=====================
    //   Function to format the hierarchy coming from MDX
    function tm1flat(tm1o, attrs, bFilter, inclAttrs) {
        //function to convert array into object
        function toObject(val, label) {
            var obj = {};
            for (var i = 0; i < label.length; i++)
                if (val[i] !== undefined && label[i] !== undefined) obj[label[i]] = val[i];
            return obj;
        }
        //========================================================
        //collect all dimensions from rows and columns
        //1) dimensions on columns
        var dims = [];
        for (var i = 0; i < tm1o.Axes[1].Hierarchies.length; i++) {
            dims.push(tm1o.Axes[1].Hierarchies[i].Name);
        }
        //2) dimensions on rows 
        for (var i = 0; i < tm1o.Axes[0].Hierarchies.length; i++) {
            dims.push(tm1o.Axes[0].Hierarchies[i].Name);
        }
        //3) OPTIONAL.filters
        if (bFilter != null && bFilter == true) {
            for (var i = 0; i < tm1o.Axes[2].Hierarchies.length; i++) {
                dims.push(tm1o.Axes[2].Hierarchies[i].Name);
            }
        }
        //4) add value to the list of dimensions
        dims.push('TM1CubeValue');

        if (inclAttrs) {
            var attributes = {};
        }
        //create auxilary arrays. rc is the main one. it contains all reference points + value as the last element.tm1fo is the array to be returned.
        var r1 = [],
            c1 = [],
            rc = [];
        var cellCount = 0;
        var tm1fo = [];

        //loop through rows
        for (var j = 0; j < tm1o.Axes[1].Tuples.length; j++) {
            for (var j1 = 0; j1 < tm1o.Axes[1].Tuples[j].Members.length; j1++) {

                if (attrs != null && tm1o.Axes[1].Tuples[j].Members[j1].Attributes[attrs[tm1o.Axes[1].Hierarchies[j1].Name]] != null) {
                    r1.push(tm1o.Axes[1].Tuples[j].Members[j1].Attributes[attrs[tm1o.Axes[1].Hierarchies[j1].Name]]);
                } else {
                    r1.push(tm1o.Axes[1].Tuples[j].Members[j1].Name);
                }
                //add dimensions attributes
                if (inclAttrs) {
                    attributes[tm1o.Axes[1].Hierarchies[j1].Name] = {};
                    for (var key in tm1o.Axes[1].Tuples[j].Members[j1].Attributes) {
                        attributes[tm1o.Axes[1].Hierarchies[j1].Name][key] = tm1o.Axes[1].Tuples[j].Members[j1].Attributes[key];
                    }
                }
            }
            //loop through columns
            for (var k = 0; k < tm1o.Axes[0].Tuples.length; k++) {
                for (var k1 = 0; k1 < tm1o.Axes[0].Tuples[k].Members.length; k1++) {
                    if (attrs != null && tm1o.Axes[0].Tuples[k].Members[k1].Attributes[attrs[tm1o.Axes[0].Hierarchies[k1].Name]] != null) {
                        c1.push(tm1o.Axes[0].Tuples[k].Members[k1].Attributes[attrs[tm1o.Axes[0].Hierarchies[k1].Name]]);
                    } else {
                        c1.push(tm1o.Axes[0].Tuples[k].Members[k1].Name);
                    }
                    if (inclAttrs) {
                        //add dimensions attributes
                        attributes[tm1o.Axes[0].Hierarchies[k1].Name] = {};

                        for (var key in tm1o.Axes[0].Tuples[k].Members[k1].Attributes) {
                            attributes[tm1o.Axes[0].Hierarchies[k1].Name][key] = tm1o.Axes[0].Tuples[k].Members[k1].Attributes[key];
                        }
                    }
                }
                rc = r1.concat(c1);
                //include filter values
                if (bFilter != null && bFilter == true) {
                    for (var m = 0; m < tm1o.Axes[2].Tuples[0].Members.length; m++) {
                        if (attrs != null && tm1o.Axes[2].Tuples[0].Members[m].Attributes[attrs[tm1o.Axes[2].Hierarchies[m].Name]] != null) {
                            rc.push(tm1o.Axes[2].Tuples[0].Members[m].Attributes[attrs[tm1o.Axes[2].Hierarchies[m].Name]]);
                        } else {
                            rc.push(tm1o.Axes[2].Tuples[0].Members[m].Name);
                        }
                    }
                }
                //add value from Cells
                rc.push(tm1o.Cells[cellCount].Value);
                tm1fo.push(toObject(rc, dims));
                if (inclAttrs) {
                    tm1fo[cellCount].Attributes = JSON.parse(JSON.stringify(attributes));
                }
                cellCount += 1;
                c1 = [];
                rc = [];
            }
            r1 = [];
            if (inclAttrs) {
                attributes = {};
            }
        }
        return tm1fo;
    };
    
    // Loop through the objects
    function loopObject(o) {
        var temp;
        var tempName;
        
        for (var i = 0; i < values.length; i++) {
            
            if (values[i].Product == o.Name && values[i].Version == 'Var %') {
                
                temp=values[i].TM1CubeValue==null?0:values[i].TM1CubeValue;
                break;
            }
        }
        o.variance=temp;
        
        if (o.hasOwnProperty('Components')) {

            for (var i = 0; i < o.Components.length; i++) {
                loopObject(o.Components[i]);
            }

        } else {

            for (var i = 0; i < values.length; i++) {

                if (values[i].Product == o.Name && values[i].Version == $scope.selections.version) {
                    o.value = values[i].TM1CubeValue;
                    break;
                }
            }
        }
    }

    // Define the colors
    var color = d3.scale.linear().domain([-0.1,0, 0.1, .3])
        .range(['red', 'orange',  'green','green' ]);

    // Initialize the size of the chart
    var margin = {
            top: 40,
            right: 0,
            bottom: 0,
            left: 0
        },
        width = parseInt(document.getElementById("chart").getBoundingClientRect().width, 10),
        height = 500 - margin.top - margin.bottom,
        formatNumber = d3.format(",d"),
        transitioning;

    var x = d3.scale.linear()
        .domain([0, width])
        .range([0, width]);

    var y = d3.scale.linear()
        .domain([0, height])
        .range([0, height]);

    var treemap = d3.layout.treemap()
        .children(function (d, depth) {
            return depth ? null : d._children;
        })
        .sort(function (a, b) {
            return a.value - b.value;
        })
        .ratio(height / width * 0.5 * (1 + Math.sqrt(5)))
        .round(false);

    var values;

    //=====================
    // Create the hierarchy
    var products;
    var mdx = '[Product].MEMBERS';
    $tm1Ui.dimensionElements('dev', 'Product', 'Code&Description', '', mdx, '', '', true, false).then(function (v) {
        products = v[0];       
        //Watch to make sure if selections update, recreate the treemap//
        $scope.$watch('selections',  function(e) {
                $scope.buildTreeMap();
            }, true);
    
        $timeout(function() {
            $window.dispatchEvent(new Event("resize"));
            
        }, 100); 
    })

    //=====================
    // Funciton to build the tree
    
    $scope.buildTreeMap = function() {        
        d3.select('#treemapchart').html('');
        var svg = d3.select("#treemapchart")
            .append("g")
            .attr("transform", "translate(" + margin.left + "," + margin.top + ")")
            .style("shape-rendering", "crispEdges");
        var grandparent = svg.append("g")
            .attr("class", "grandparent");
        grandparent.append("rect")
            .attr("y", -margin.top)
            .attr("width", width)
            .attr("height", margin.top);
        grandparent.append("text")
            .attr("x", 6)
            .attr("y", 6 - margin.top)
            .attr("dy", "0.75em");
        // MDX used to get the values
        var mdx1 = 'select {[Product].MEMBERS} on 0, {[Version].[' + $scope.selections.version + '],[Version].[var %]} on 1 from [Retail] where ([Year].[' + $scope.selections.year + '],[Period].[Year],[Currency].[Local],[Region].[' + $scope.selections.region + '],[Retail Measure].[' + $scope.selections.measure + '])';
            
        $tm1Ui.cubeExecuteMdx('dev', mdx1).then(function (data) {values = tm1flat(data, {'Version': 'Description'}, true);
            
        loopObject(products);   
        v = products;
        initialize(v);
        accumulate(v);
        layout(v);
        display(v);

        function initialize(root) {
            root.x = root.y = 0;
            root.dx = width;
            root.dy = height;
            root.depth = 0;
        }

        function accumulate(d) {
            return (d._children = d.Components) ? d.value = d.Components.reduce(function (p, v) {
                return p + accumulate(v);
            }, 0) : d.value;
        }

        function layout(d) {
            if (d._children) {
                treemap.nodes({
                    _children: d._children
                });
                d._children.forEach(function (c) {
                    c.x = d.x + c.x * d.dx;
                    c.y = d.y + c.y * d.dy;
                    c.dx *= d.dx;
                    c.dy *= d.dy;
                    c.parent = d;
                    layout(c);
                });
            }
        }

        function display(d) {
            grandparent
                .datum(d.parent)
                .on("click", transition)
                .select("text")
                .text(name(d));

            var g1 = svg.insert("g", ".grandparent")
                .datum(d)
                .attr("class", "depth");

            var g = g1.selectAll("g")
                .data(d._children)
                .enter().append("g");

            g.filter(function (d) {
                    return d._children;
                })
                .classed("children", true)
                .on("click", transition);

            g.selectAll(".child")
                .data(function (d) {
                    return d._children || [d];
                })
                .enter().append("rect")
                .attr("class", "child")
                .call(rect);

            g.append("rect")
                .attr("class", "parent")
                .call(rect)

                .append("title")
                .text(function (d) {
                    return d.Attributes['Code&Description'];
                    //return formatNumber(d.value);
                });

            g.append("text")
                .attr("dy", "0.75em")
                .text(function (d) {
                    return d.Name + ' (A: ' + Math.round(d.value) + '. V: '+Math.round(d.variance*100,2)+'%)';
                })
                .call(text);

            function transition(d) {
                if (transitioning || !d) return;
                transitioning = true;

                var g2 = display(d),
                    t1 = g1.transition().duration(750),
                    t2 = g2.transition().duration(750);

                // Update the domain only after entering new elements.
                x.domain([d.x, d.x + d.dx]);
                y.domain([d.y, d.y + d.dy]);

                // Enable anti-aliasing during the transition.
                svg.style("shape-rendering", null);

                // Draw child nodes on top of parent nodes.
                svg.selectAll(".depth").sort(function (a, b) {
                    return a.depth - b.depth;
                });

                // Fade-in entering text.
                g2.selectAll("text").style("fill-opacity", 0);

                // Transition to the new view.
                t1.selectAll("text").call(text).style("fill-opacity", 0);
                t2.selectAll("text").call(text).style("fill-opacity", 1);
                t1.selectAll("rect").call(rect);
                t2.selectAll("rect").call(rect);

                // Remove the old node when the transition is finished.
                t1.remove().each("end", function () {
                    svg.style("shape-rendering", "crispEdges");
                    transitioning = false;
                });
            }

            return g;
        }

        function text(text) {
            text.attr("x", function (d) {
                    return x(d.x) + 6;
                })
                .attr("y", function (d) {
                    return y(d.y) + 6;
                });
        }

        function rect(rect) {
            rect.attr("x", function (d) {
                    return x(d.x);
                })
                .attr("y", function (d) {
                    return y(d.y);
                })
                .attr("width", function (d) {
                    //console.log(x(d.x + d.dx) - x(d.x));
                    return x(d.x + d.dx) - x(d.x);
                })
                .attr("height", function (d) {
                    return y(d.y + d.dy) - y(d.y);
                }).style('fill', function (d, i) {
                    
                    //return 'green';
                    return color(d.variance);
                });
        }

        function name(d) {
            return d.parent ? name(d.parent) + "." + d.Attributes['Code&Description'] + ' (' + Math.round(d.value) + ')' : d.Attributes['Code&Description'] + ' (' + Math.round(d.value) + ')';
        }

        });
    }
    // END Function Build treemap //

    //RESIZE Treemap FUNCTION START //
    $(window).resize(function(){ 

        document.getElementById("treemapchart").style.height = parseInt(($window.innerHeight) - (document.getElementById("treemapchart").getBoundingClientRect().top)) +"px";
        width = parseInt(document.getElementById("chart").getBoundingClientRect().width, 10);
        height = parseInt(($window.innerHeight) - (document.getElementById("treemapchart").getBoundingClientRect().top)) - margin.top - margin.bottom;
       
        x = d3.scale.linear()
            .domain([0, width])
            .range([0, width]);

        y = d3.scale.linear()
            .domain([0, height])
            .range([0, height]);

        treemap = d3.layout.treemap()
            .children(function (d, depth) {
                return depth ? null : d._children;
            })
            .sort(function (a, b) {
                return a.value - b.value;
            })
            .ratio(height / width * 0.5 * (1 + Math.sqrt(5)))
            .round(false); 
            
            $scope.buildTreeMap();
    });

    //Watch if the navigation open //
    $scope.$watch('navisopened', function(e) {
        $timeout(function() {
        
            $window.dispatchEvent(new Event("resize"));
        }, 300); 
    }, true);

    //   END TREEMAP  //
    //################//
}]);